#version 130
#extension GL_EXT_gpu_shader3 : enable
#extension GL_EXT_gpu_shader4 : enable

uniform sampler2D textureColorCurve;
uniform sampler2D textureImage[3];

//uniform int useTexAlpha;
uniform int dissolveMode;
uniform float boldness;

attribute vec2 colorCurveST;
attribute float filterAlpha;
attribute int paramBit;

out vec3 positionIn;
out vec3 normalIn;
out vec4 GLSubPosition;
out vec4 colorIn;
out vec4 colorDiffuseIn;
out vec4 colorSpecularIn;
flat out int blendIn;
flat out int textureImageIndexIn; // valid 0-3 not use if 4;

void main(void)
{
    gl_TexCoord[0] = gl_TextureMatrix[0] * gl_MultiTexCoord0;
	
    vec4 baseColor = texture2D(textureColorCurve, colorCurveST);
    baseColor[3] *= filterAlpha;

    vec4 colorDiffuseBase = texture2D(textureColorCurve, colorCurveST+vec2(0, 0.3333));
    vec4 colorSpecularBase = texture2D(textureColorCurve, colorCurveST+vec2(0, 0.6666));
    
    float alphaOrg = baseColor.a;
    float alpha0 = alphaOrg;

    if (dissolveMode == 1)
	if (alphaOrg < 0.015)
	    alpha0 = 1;
    
    if (dissolveMode == 2)
	alpha0 = 1;
    
    colorIn = vec4(baseColor.rgb * alpha0, alphaOrg);
    colorDiffuseIn = vec4(colorDiffuseBase.rgb * alpha0, alphaOrg);
    colorSpecularIn = vec4(colorSpecularBase.rgb * alpha0, alphaOrg);
    
    positionIn = vec3(gl_ModelViewMatrix * gl_Vertex);
    normalIn = gl_NormalMatrix * gl_Normal;

    gl_Position = ftransform();
    float distance = gl_Position.z;
    GLSubPosition = gl_Vertex + vec4(gl_Normal, 0) * 0.2 * boldness * sqrt(distance);
    GLSubPosition = gl_ModelViewProjectionMatrix * GLSubPosition;
    
    blendIn = paramBit % 2;

    if ((paramBit / 8) % 2 == 0)
	textureImageIndexIn = 4;
    else
	textureImageIndexIn = (paramBit / 16) % 4;
}
