//#extension GL_EXT_gpu_shader4 : enable

uniform sampler3D textureVolume;
uniform sampler1D textureCC;
uniform int lightNumber;
uniform vec3 ambientClr;
uniform vec3 diffuseClr;
uniform vec3 specularClr;
uniform int shininess;
uniform int forceDiscard0;
uniform int trim;

varying vec3 position;

//##FUNC_VOLUME_MASK_BEGIN##
bool GetMask(vec3 texCoord){
    return true;
}
//##FUNC_VOLUME_MASK_END##

//##FUNC_VOLUME_COORD_BEGIN##
//#define GTRIM 1
vec3 GetDeformCoord(vec3 vecOrg){
    return vecOrg;
}

#define GUSEMATRIX 0
mat3 GetDeformNormalMatrix(vec3 texCoord, vec3 deformedCoord){
    return mat3(1, 0, 0, 0, 1, 0, 0, 0, 1);
}
//##FUNC_VOLUME_COORD_END##

//##FUNC_SPOT_EFFECT##

void main (void)
{
    gl_FragColor = vec4(0, 0, 0 ,0);
    
    vec3 texCoord = vec3(gl_TexCoord[0]);
    vec3 deformedCoord = GetDeformCoord(texCoord);
    if (GetMask(deformedCoord) == false) discard;

#if GTRIM
    if (deformedCoord.x < 0 || deformedCoord.x > 1) discard;
    if (deformedCoord.y < 0 || deformedCoord.y > 1) discard;
    if (deformedCoord.z < 0 || deformedCoord.z > 1) discard;
#else
    if (trim == 1){
	if (deformedCoord.x < 0 || deformedCoord.x > 1) discard;
	if (deformedCoord.y < 0 || deformedCoord.y > 1) discard;
	if (deformedCoord.z < 0 || deformedCoord.z > 1) discard;
    }
#endif

    vec4 texColor = texture3D(textureVolume, deformedCoord);
    float value = texColor[0];
    if (forceDiscard0 == 1 && value == 0) discard;
    
    vec4 texColor2 = texture1D(textureCC, value);
    if (texColor2.a == 0) discard;

    //ivec3 texSize = textureSize3D(textureVolume, 0);

    int flatData = 0;
    //int kSize = 1; // kernelSize
    vec3 gradientL;

#if GUSEMATRIX
    vec3 gradient = vec3(textureOffset(textureVolume, deformedCoord, ivec3(-1, 0, 0)).r - textureOffset(textureVolume, deformedCoord, ivec3(1, 0, 0)).r,
		    textureOffset(textureVolume, deformedCoord, ivec3(0, -1, 0)).r - textureOffset(textureVolume, deformedCoord, ivec3(0, 1, 0)).r,
		    textureOffset(textureVolume, deformedCoord, ivec3(0, 0, -1)).r - textureOffset(textureVolume, deformedCoord, ivec3(0, 0, 1)).r);
    
    mat3 deformedMatrix = GetDeformNormalMatrix(texCoord, deformedCoord);
    vec3 deformedGradient = deformedMatrix * gradient;
    
    gradientL = vec3(gl_NormalMatrix * mat3(gl_TextureMatrixInverse[0]) * deformedGradient);
#else
    //ivec3 texSize = textureSize3D(textureVolume, 0);
    float delta = 0.01;
    
    vec3 deformedCoord_x1 = GetDeformCoord(texCoord + vec3(+delta, 0, 0));
    vec3 deformedCoord_x0 = GetDeformCoord(texCoord + vec3(-delta, 0, 0));
    vec3 deformedCoord_y1 = GetDeformCoord(texCoord + vec3(0, +delta, 0));
    vec3 deformedCoord_y0 = GetDeformCoord(texCoord + vec3(0, -delta, 0));
    vec3 deformedCoord_z1 = GetDeformCoord(texCoord + vec3(0, 0, +delta));
    vec3 deformedCoord_z0 = GetDeformCoord(texCoord + vec3(0, 0, -delta));
    
    gradientL = -vec3
	(texture3D(textureVolume, deformedCoord_x1).r - 
	 texture3D(textureVolume, deformedCoord_x0).r,
	 texture3D(textureVolume, deformedCoord_y1).r - 
	 texture3D(textureVolume, deformedCoord_y0).r,
	 texture3D(textureVolume, deformedCoord_z1).r - 
	 texture3D(textureVolume, deformedCoord_z0).r); 
#endif
    
    if (gradientL[0] == 0 && gradientL[1] == 0 && gradientL[2] == 0)
	flatData = 1;
    
    vec3 fnormal = normalize(gradientL);
    vec3 viewVec = -normalize(position);
    
    for (int i = 0; i < lightNumber; i++){
	float diffuse = 0;
	float specular = 0;
	if (flatData == 0){
	    vec3 lightVec = gl_LightSource[i].position.xyz - position * gl_LightSource[i].position[3];
	    float dis = length(lightVec);
	    lightVec = normalize(lightVec);
	    
	    diffuse = dot(lightVec, fnormal) * 4;
	    if (diffuse < 0) diffuse = 0;
	
	    float spotAngleCos = dot(normalize(gl_LightSource[i].spotDirection),
				     -lightVec);
	    float lightFactor = GetSpotEffect(i, spotAngleCos);
	    
	    lightFactor /= (gl_LightSource[i].constantAttenuation +
			    gl_LightSource[i].linearAttenuation * dis +
			    gl_LightSource[i].quadraticAttenuation * dis * dis);
	    
	    //˃xNg 
	    vec3 reflectVec = reflect(-lightVec, fnormal);
	    specular = pow(max(dot(viewVec, reflectVec), 0.0), float(shininess)) * lightFactor;
	    diffuse *= lightFactor;
	}
	
	vec4 color = texColor2 * gl_Color;
	
	gl_FragColor[0] +=
	    color.r * diffuseClr.r * diffuse * gl_LightSource[i].diffuse.r +
	    color.r * ambientClr.r * gl_LightSource[i].ambient.r + 
	    specularClr.r * specular * gl_LightSource[i].specular.r;
		
	gl_FragColor[1] += 
	    color.g * diffuseClr.g * diffuse * gl_LightSource[i].diffuse.g +
	    color.g * ambientClr.g * gl_LightSource[i].ambient.g +
	    specularClr.g * specular * gl_LightSource[i].specular.g;
	
	gl_FragColor[2] += 
	    color.b * diffuseClr.b * diffuse * gl_LightSource[i].diffuse.b +
	    color.b * ambientClr.b * gl_LightSource[i].ambient.b + 
	    specularClr.b * specular * gl_LightSource[i].specular.b;
    }

    gl_FragColor[3] = texColor2.a * gl_Color.a;
    
    return;
}
